#include <SoftwareSerial.h>
#include <string.h>

SoftwareSerial mySerial(10, 9);
const char *ID = "";
const char *sub = "";
const char *pub = "";

const char *mqtt_server = "mqtt.waveshare.cloud";
char connect_cmd[100];
char dataLength[20];

bool flag = true;

void send_at(const char *p_char) {
  char r_buf[100];

  mySerial.println(p_char);
  delay(1000);

  size_t i = mySerial.available();
  for (size_t j = 0; j < i; j++) {
    r_buf[j] = mySerial.read();
  }
  r_buf[i] = '\0';

  char *p_response = strstr(r_buf, "\r\n");
  if (p_response != NULL) {
    p_response += 2;
    if (strstr(p_response, "+CREG: 0,") != NULL) {
      char *p_status = strstr(p_response, "+CREG: 0,") + 9;
      int status = atoi(p_status);
      if (status == 6) {
        flag = false;
        Serial.println();
        Serial.println("NetWork Connected");
      }
    }
    if (strstr(p_response, "+CMQTTCONNECT: 0,") != NULL) {
      char *p_status = strstr(p_response, "+CMQTTCONNECT: 0,") + 17;
      int status = atoi(p_status);
      if (status == 0) {
        flag = false;
        Serial.println();
        Serial.println("Mqtt Connected");
      } else if (status == 23) {
        flag = false;
        Serial.println();
        Serial.println("Mqtt is already Connected");
      }
    }
  }
}

void input_message(const char *p_char, const char *p_data) {
  char r_buf[100];

  mySerial.println(p_char);
  delay(200);
  mySerial.print(p_data);
  delay(1000);

  size_t i = mySerial.available();
  for (size_t j = 0; j < i; j++) {
    r_buf[j] = mySerial.read();
  }
  r_buf[i] = '\0';

  char *p_response = strstr(r_buf, "\r\n");
  if (p_response != NULL) {
    p_response += 2;

    if (strstr(p_response, "OK") != NULL) {
      Serial.println("Write OK");
    } else if (strstr(p_response, "ERROR") != NULL) {
      Serial.println("Write ERROR");
    }
  }
}

void getResponseData() {
  static char buffer[200];
  static int bufferIndex = 0;

  while (mySerial.available()) {
    char c = mySerial.read();
    if (c == '\r') {
      continue;
    }
    if (c == '\n') {
      buffer[bufferIndex] = '\0';
      Serial.println(buffer);
      bufferIndex = 0;
    } else {
      if (bufferIndex < sizeof(buffer) - 1) {
        buffer[bufferIndex++] = c;
      }
    }
  }
}


bool isSoftwareSerialAvailable() {
  return mySerial.available() > 0;
}

void setup() {
  pinMode(8, OUTPUT);
  digitalWrite(8, HIGH);
  delay(400);
  digitalWrite(8, LOW);
  Serial.begin(9600);
  mySerial.begin(115200);
  while (!isSoftwareSerialAvailable()) {
    Serial.println("Software Serial Device Not Found! Retrying...");
    delay(2000);
  }
  Serial.print("SIM7028 GET Network...");
  while (flag) {
    Serial.print(".");
    send_at("AT+CREG?");
  }
  flag = true;
  send_at("AT");
  send_at("AT+CSQ");
  Serial.println("MQTT START!");
  send_at("AT+CMQTTSTART");
  Serial.print("Set Client ID:");
  snprintf(connect_cmd, sizeof(connect_cmd), "AT+CMQTTACCQ=0,%s,0", ID);
  Serial.println(connect_cmd);
  send_at(connect_cmd);
  snprintf(connect_cmd, sizeof(connect_cmd), "AT+CMQTTCONNECT=0,tcp://%s:1883,20,1", mqtt_server);
  Serial.print("Connect to tcp://");
  Serial.println(mqtt_server);
  while (flag) {
    Serial.print(".");
    send_at(connect_cmd);
    delay(1000);
  }

  snprintf(dataLength, sizeof(dataLength), "%d", strlen(pub));
  snprintf(connect_cmd, sizeof(connect_cmd), "AT+CMQTTTOPIC=0,%s", dataLength);
  input_message(connect_cmd, pub);
  delay(1000);

  snprintf(dataLength, sizeof(dataLength), "%d", strlen(pub));
  snprintf(connect_cmd, sizeof(connect_cmd), "AT+CMQTTSUB=0,%s,0", dataLength);
  input_message(connect_cmd, sub);

  delay(2000);
  char *msgdata = "This is Sim7028 with Arduino!";
  snprintf(dataLength, sizeof(dataLength), "%d", strlen(msgdata));
  snprintf(connect_cmd, sizeof(connect_cmd), "AT+CMQTTPAYLOAD=0,%s", dataLength);
  input_message(connect_cmd, msgdata);
  delay(1000);
}

void loop() {
  
  delay(2000);
  getResponseData();
  send_at("AT+CMQTTPUB=0,0,60");
}
