#include <ArduinoJson.h>
TaskHandle_t TASK_HandleOne = NULL;

const char *ID = "";
const char *sub = "";
const char *pub = "";

const char *mqtt_server = "mqtt.waveshare.cloud";

bool isSerial2Available = true;
String rev;
char connect_cmd[100];
char dataLength[20];

void getDevice();
void TASK_ONE(void *param);

void getDevice() {
  Serial2.println("AT");
  delay(100);
  rev = Serial2.readString();
  char *p_response = strstr(rev.c_str(), "\r\n");
  if (p_response != NULL) {
    p_response += 2;
    if (strstr(p_response, "OK") != NULL) {
      Serial.println("Got Sim7208!");
      isSerial2Available = false;
    }
  } else {
    Serial.println("Serial Device Not Found! Retrying...");
  }
}

void SentMessage(const char *p_char) {
  char r_buf[100];
  Serial2.println(p_char);
  delay(1000);

  size_t i = Serial2.available();
  for (size_t j = 0; j < i; j++) {
    r_buf[j] = Serial2.read();
  }
  r_buf[i] = '\0';

  char *p_response = strstr(r_buf, "\r\n");
  if (p_response != NULL) {
    p_response += 2;
    if (strstr(p_response, "+CREG: 0,") != NULL) {
      char *p_status = strstr(p_response, "+CREG: 0,") + 9;
      int status = atoi(p_status);
      if (status == 6) {
        isSerial2Available = false;
        Serial.println();
        Serial.println("NetWork Connected");
      }
    }
    if (strstr(p_response, "+CMQTTCONNECT: 0,") != NULL) {
      char *p_status = strstr(p_response, "+CMQTTCONNECT: 0,") + 17;
      int status = atoi(p_status);
      if (status == 0) {
        isSerial2Available = false;
        Serial.println();
        Serial.println("Mqtt Connected");
      } else if (status == 23) {
        isSerial2Available = false;
        Serial.println();
        Serial.println("Mqtt is already Connected");
      }
    }
  }
}

void input_message(const char *p_char, const char *p_data) {
  char r_buf[100];

  Serial2.println(p_char);
  delay(200);
  Serial2.print(p_data);
  delay(1000);

  size_t i = Serial2.available();
  for (size_t j = 0; j < i; j++) {
    r_buf[j] = Serial2.read();
  }
  r_buf[i] = '\0';

  char *p_response = strstr(r_buf, "\r\n");
  if (p_response != NULL) {
    p_response += 2;

    if (strstr(p_response, "OK") != NULL) {
      Serial.println("Write OK");
    } else if (strstr(p_response, "ERROR") != NULL) {
      Serial.println("Write ERROR");
    }
  }
}

void getResponseData() {
  static char buffer[200];
  static int bufferIndex = 0;

  while (Serial2.available()) {
    char c = Serial2.read();
    if (c == '\r') {
      continue;
    }
    if (c == '\n') {
      buffer[bufferIndex] = '\0';
      if (strncmp(buffer, "+CMQTTPUB: 0,0", 14) != 0 && strcmp(buffer, "OK") != 0) {
        if (!(bufferIndex == 0 && c == '\n')) {
          if (strstr(buffer, "{") != NULL) {
            // Serial.print(buffer);
            StaticJsonDocument<200> jsonDocument;
            DeserializationError error = deserializeJson(jsonDocument, buffer);
            if (error == DeserializationError::Ok) {
              // Access the "data" object and get the "led" value
              int ledValue = jsonDocument["data"]["led"];
              if (ledValue == 1) {
                digitalWrite(5, HIGH);
              } else if (ledValue == 0) {
                digitalWrite(5, LOW);
              }
            } else {
              Serial.println("JSON parsing failed!");
            }
          }
        }
      }
      bufferIndex = 0;
    } else {
      if (bufferIndex < sizeof(buffer) - 1) {
        buffer[bufferIndex++] = c;
      }
    }
  }
}


void setup() {
  pinMode(4, OUTPUT);
  pinMode(5, OUTPUT);
  digitalWrite(4, HIGH);
  digitalWrite(5, LOW);
  delay(400);
  digitalWrite(4, LOW);
  Serial.begin(9600);
  Serial2.begin(115200);

  while (isSerial2Available) {
    getDevice();
  }

  isSerial2Available = true;

  Serial.print("Get Network...");
  while (isSerial2Available) {
    Serial.print(".");
    SentMessage("AT+CREG?");
    delay(1000);
  }

  Serial2.println("AT+CSQ");
  Serial2.println("AT+CMQTTSTART");
  snprintf(connect_cmd, sizeof(connect_cmd), "AT+CMQTTACCQ=0,%s,0", ID);
  Serial2.println(connect_cmd);
  snprintf(connect_cmd, sizeof(connect_cmd), "AT+CMQTTCONNECT=0,tcp://%s:1883,20,1", mqtt_server);
  Serial.print("Connect to tcp://");
  Serial.println(mqtt_server);
  isSerial2Available = true;
  while (isSerial2Available) {
    Serial.print(".");
    SentMessage(connect_cmd);
    delay(1000);
  }
  snprintf(dataLength, sizeof(dataLength), "%d", strlen(pub));
  snprintf(connect_cmd, sizeof(connect_cmd), "AT+CMQTTTOPIC=0,%s", dataLength);
  input_message(connect_cmd, pub);
  delay(1000);

  snprintf(dataLength, sizeof(dataLength), "%d", strlen(pub));
  snprintf(connect_cmd, sizeof(connect_cmd), "AT+CMQTTSUB=0,%s,0", dataLength);
  input_message(connect_cmd, sub);

  delay(2000);
  char *msgdata = "This is Sim7028 with Arduino!";
  snprintf(dataLength, sizeof(dataLength), "%d", strlen(msgdata));
  snprintf(connect_cmd, sizeof(connect_cmd), "AT+CMQTTPAYLOAD=0,%s", dataLength);
  input_message(connect_cmd, msgdata);
  delay(1000);

  xTaskCreate(
    TASK_ONE,
    "TaskOne",
    12 * 1024,
    NULL,
    1,
    &TASK_HandleOne);
}

void loop() {
  getResponseData();
  // char *msgdata = "{"data":{"temp":21,"hum":34}}";
  // snprintf(dataLength, sizeof(dataLength), "%d", strlen(msgdata));
  // snprintf(connect_cmd, sizeof(connect_cmd), "AT+CMQTTPAYLOAD=0,%s", dataLength);
  // input_message(connect_cmd, msgdata);
}

void TASK_ONE(void *param) {
  while (1) {
    SentMessage("AT+CMQTTPUB=0,0,60");
    vTaskDelay(pdMS_TO_TICKS(5000));
  }
}